#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-07 22:21
# Author : fyt
# File   : file.py

import os

def create_data_file():

    response_path = './dataFile'

    if not os.path.isdir(response_path):

       os.makedirs(response_path)

       with open(response_path + os.sep + 'response.json','w+') as json_file:

           json_file.write('{}')

    return response_path + os.sep + 'response.json'


def find_obj_yaml_file():

    for dirpath, dirnames, filenames in os.walk('./'):
        if len(filenames):
           for index ,file_name in enumerate(filenames):
               if bool(os.path.splitext(file_name).count('.yaml')):
                   return os.path.join(dirpath,file_name)

    return None


if __name__ == '__main__':
    import pytest

    def data():
        return {"key":"value"}

    @pytest.mark.parametrize('dic',data())



    def test_dic(dic):
        print(dic)




#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-07 22:22
# Author : fyt
# File   : apiAssert.py

from urllib import parse
import operator, json, re, requests

from ytApiTest import apiReq
from ytApiTest import parsingData


class assert_error_info(Exception):
    '''
    自定义错误类，输出断言错误信息
    '''

    def __init__(self, errorInfo):
        self.errorInfo = str(errorInfo)

    def __str__(self):
        return self.errorInfo


def fmt_assert_info(body, assertValue, differenceValue):
    '''
    格式化错误信息
    :param body: 接口返回值
    :param assertValue: 断言值
    :param differenceValue: 断言失败值
    :return: 组合错误信息
    '''

    if isinstance(body, requests.Response):
        data = ''
        url = body.request.url
        if body.request.body != None:
            data = parse.unquote(body.request.body)
        interface_info = url + data

        info = 'ErrorInfo: {errorInfo}\n\n' \
               'InterfaceInfo: {interfaceInfo}\n\n' \
               'assertValue: {assertValue}'.format(errorInfo=differenceValue,
                                                   interfaceInfo=interface_info,
                                                   assertValue='')

        apiReq.send_ding_talk_info(title='接口断言失败',
                                   text=info)
        return info
    else:
        return None


def assert_url_code(body, assertValue):
    '''
    断言请求状态
    :return:
    '''

    assert operator.eq(body.status_code, assertValue), fmt_assert_info(body, assertValue, '状态码不等于200')


def assert_body_include_value(body=None, assertValue=None):
    '''
    断言body 是否包含 value
    :return:
    '''
    if body == None and assertValue == None:
        return

    body_str = json.dumps(parsingData.parser_response(body), ensure_ascii=False)
    assert_str = json.dumps(assertValue, ensure_ascii=False)
    if len(assert_str.split(',')) == 1:
        assert remove_special_characters(body_str).split(',').count(
            remove_special_characters(assert_str).split(',')[0]), fmt_assert_info(body, assertValue, assert_str)

        return

    matching_value = find_response_assert_value(assert_value=assert_str,
                                                response_value=body_str)
    if operator.eq(None, matching_value):
        assert_error_info(fmt_assert_info(body, assertValue, '返回值不包含断言数据'))
        print('找不到断言数据', matching_value)
        return assert_error_info('返回值不包含断言数据')
    response_str_set = set(matching_value)
    assert_str_set = set(assert_str.replace(' ', '').split(','))
    assert_difference_info = list(assert_str_set.difference(response_str_set))
    response_difference_info = list(response_str_set.difference(assert_str_set))
    response_difference_info.sort()
    assert_difference_info.sort()
    error_info = {'ASSERT': '{assert_difference_info}'.format(assert_difference_info=assert_difference_info),
                  '************': '***********', 'RESPONSE': '{response_difference_info}'.format(
            response_difference_info=response_difference_info)}

    assert operator.eq(len(assert_difference_info), 0), assert_error_info(
        fmt_assert_info(body, assertValue, error_info))
    assert_response_url_status(body_str)


def assert_body_ep_value(body=None, assertValue=None):
    '''
    断言body 是否与value完全相等
    :return:
    '''
    if body == None and assertValue == None:
        return

    body_str = remove_special_characters(json.dumps(parsingData.parser_response(body), ensure_ascii=False))
    assert_str = remove_special_characters(json.dumps(assertValue, ensure_ascii=False))

    response_str_set = set(body_str.split(','))
    assert_str_set = set(assert_str.split(','))
    assert_difference_info = list(assert_str_set.difference(response_str_set))
    response_difference_info = list(response_str_set.difference(assert_str_set))
    response_difference_info.sort()
    assert_difference_info.sort()
    error_info = {'ASSERT': '{assert_difference_info}'.format(assert_difference_info=response_difference_info),
                  '************': '***********', 'RESPONSE': '{response_difference_info}'.format(
            response_difference_info=assert_difference_info)}

    assert operator.eq(len(assert_difference_info), 0), assert_error_info(
        fmt_assert_info(body, assertValue, error_info))
    assert_response_url_status(body_str)


def assert_response_url_status(response_str):
    url_data_list = re.findall("[a-zA-z]+://[^\s]*", response_str)
    if len(url_data_list):
        for index, url in enumerate(url_data_list):
            url = str(url).replace("\"", '').replace(',', '')
            requests.packages.urllib3.disable_warnings()
            r = requests.get(remove_special_characters(url), verify=False)
            r.url
            assert_url_code(r, 200)


def splice_regula_expression(dic):
    '''
    生成匹配断言正则表达式
    :param dic: 断言数据
    :return:
    '''

    if isinstance(dic, dict):
        assert_data_str = remove_special_characters(json.dumps(dic, ensure_ascii=False))
        ex_star_str = assert_data_str[:assert_data_str.find(':')]
        ex_end_str = assert_data_str[assert_data_str.rfind(':'):]
        pattern = ex_star_str + '.*' + ex_end_str

        return pattern


def remove_special_characters(string):
    '''
    删除特殊字符{,[,],}
    :param string:
    :return:
    '''
    if isinstance(string, str):
        return string.replace('{', '').replace('}', '').replace('[', '').replace(']', '').replace(' ', '')


def interception_json_assert_value(assert_value_str: str, response_str: str):
    response_str = response_str[1:-1]

    if assert_value_str.endswith("]}"):

        sta_str = assert_value_str[:assert_value_str.index(':')].replace('{', '')
        end_str = str(assert_value_str[assert_value_str.rfind(',') + 1:]).replace('}', '')

        if operator.ne(response_str.find(sta_str), -1) and operator.ne(response_str.find(end_str), -1):
            value = response_str[response_str.index(sta_str):response_str.index(end_str) + len(end_str)]

            return value

    else:

        sta_str = assert_value_str[:assert_value_str.index(':')].replace('{', '')
        end_str = str(assert_value_str[:assert_value_str.rfind(':') + 1]).replace('}', '')
        end_str = end_str[end_str.rfind(',') + 1:]

        if operator.ne(response_str.find(sta_str), -1) and operator.ne(response_str.find(end_str), -1):
            end_str_index = response_str.index(',', response_str.index(end_str))
            sta_str_index = response_str.index(sta_str)

            return response_str[sta_str_index:end_str_index]


def find_response_assert_value(assert_value: str, response_value: str):
    # 我也不知道我写了什么 -_-!!!,反正有个隐藏bug，就是头部尾部都找不到的时候没有处理~~，他妈的有问题再改就是了
    response_value_list = response_value.translate(response_value.maketrans('{}[]', '    ')).replace(' ', '').split(',')

    assert_value_list = assert_value.translate(response_value.maketrans('{}[]', '    ')).replace(' ', '').split(',')

    start_str = assert_value_list[0]
    end_str = assert_value_list[-1]

    try:

        if response_value_list.count(start_str) and response_value_list.count(end_str):
            # 处理头尾都存在情况
            start_str_index = response_value_list.index(start_str)
            end_str_index = response_value_list.index(end_str)

            interception_list_response_value = response_value.split(',')[start_str_index:end_str_index + 1]
            cutting_value = assert_value.split(',')
            replace_first_value = interception_colon_before_value(cutting_value[0])
            new_interception_list_response_first_value = interception_list_response_value[0].replace(
                interception_colon_before_value(interception_list_response_value[0]), replace_first_value)
            new_interception_list_response_last_value = interception_list_response_value[-1].replace(
                interception_colon_before_value(interception_list_response_value[-1], True),
                interception_colon_before_value(cutting_value[-1], True))

            interception_list_response_value[0] = new_interception_list_response_first_value
            interception_list_response_value[-1] = new_interception_list_response_last_value
            interception_list_response_value = [v.replace(' ', '') for v in interception_list_response_value]
            return interception_list_response_value

            # print(','.join(interception_list_response_value))
        elif response_value_list.count(start_str) and not response_value_list.count(end_str):
            # 处理只有头部匹配情况

            interception_list_response_value = ','.join(
                response_value.replace(' ', '').split(',')[response_value_list.index(start_str) + 1:])[
                                               :len(','.join(assert_value.replace(' ', '').split(',')[1:]))].split(',')
            interception_list_response_value.insert(0, assert_value.split(',')[0])

            if assert_value.split(',')[-1].find(':') != -1:
                interception_list_response_value[-1] = interception_list_last_special_chars(
                    assert_value.split(',')[-1].split(':')[-1], interception_list_response_value[-1])
            else:
                interception_list_response_value[-1] = interception_list_last_special_chars(assert_value.split(',')[-1],
                                                                                            interception_list_response_value[
                                                                                                -1])

            return interception_list_response_value
        elif not response_value_list.count(start_str) and response_value_list.count(end_str):
            # 处理只有头部匹配情况

            interception_response_value = ','.join(
                response_value.replace(' ', '').split(',')[:response_value_list.index(end_str)])

            interception_list_response_value = ('{' + interception_response_value[(len(
                interception_response_value) - len(','.join(assert_value.replace(' ', '').split(',')[:-1]))):]).split(
                ',')

            interception_list_response_value.insert(len(interception_list_response_value),
                                                    assert_value.replace(' ', '').split(',')[-1])

            interception_list_response_value[0] = interception_list_first_special_chars(
                assert_first_value=assert_value.split(',')[0],
                response_first_value=interception_list_response_value[0])
            interception_list_response_value = [v.replace(' ', '') for v in interception_list_response_value]
            return interception_list_response_value

    except ValueError as e:
        print(e)


def interception_list_last_special_chars(assert_last_value: str, response_lase_value: str):
    one_index = assert_last_value.find('}')
    two_index = assert_last_value.find(']')
    if one_index != -1 and two_index != -1:

        value = assert_last_value[one_index if one_index < two_index else two_index:]
        response_lase_value = response_lase_value.translate(response_lase_value.maketrans('{}[]', '    ')).replace(' ',
                                                                                                                   '')

        return response_lase_value.translate(response_lase_value.maketrans('{}[]', '    ')).replace(' ', '') + value

    elif one_index == -1:

        value = assert_last_value[two_index:]

        return response_lase_value.translate(response_lase_value.maketrans('{}[]', '    ')).replace(' ', '') + value

    elif two_index == -1:

        value = assert_last_value[one_index:]

        return response_lase_value.translate(response_lase_value.maketrans('{}[]', '    ')).replace(' ', '') + value


def special_last_chars_index(chars: str):
    split_value = chars.split(':')[1]
    one_index = split_value.find('}')
    two_index = split_value.find(']')
    index = one_index if one_index > two_index else two_index
    return index - 1


def interception_list_first_special_chars(assert_first_value: str, response_first_value: str):
    return assert_first_value[:special_chars_index(assert_first_value)] + response_first_value[
                                                                          special_chars_index(response_first_value):]


def special_chars_index(chars: str):
    split_value = chars.split(':')[0]
    one_index = split_value.rfind('{')
    two_index = split_value.rfind('[')
    index = one_index if one_index > two_index else two_index

    return index + 1


def interception_colon_before_value(interception_value: str, is_last=False):
    if is_last:
        return interception_value[interception_value.rfind(':'):]

    return interception_value[:interception_value.rfind(':')]


if __name__ == '__main__':
    s = '{{"userExtInfo":{"userId":2661076'
    a = '{"userExtInfo":{"userId":2661076'
    print(interception_list_first_special_chars(s, a))


#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-09 21:27
# Author : fyt
# File   : configKey.py

DING_TALK_URL = 'DING_TALK_URL' #钉钉URL
OBJECT_HOST = 'OBJECT_HOST' #项目host
ACCOUNT_URL = 'ACCOUNT_URL' #账号host
ACCOUNT_DATA = 'ACCOUNT_DATA'

class YAML_KEY():
    URL = 'url'
    DES = 'des'
    REQ_DATA = 'req_data'
    ASSERT_DATA = 'ast_data'

if __name__ == '__main__':
    s = '123'
    print(s[s.index('3')+1:])


#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-07 22:22
# Author : fyt
# File   : parsingData.py

import yaml, json, os, jsonpath, operator, requests
from ytApiTest.file import create_data_file
from ytApiTest import configKey, file
from urllib import parse


class YamlSingleton():
    _obj = None
    _init_flag = True
    yaml_data = None

    def __new__(cls, *args, **kwargs):

        if YamlSingleton._obj == None:
            YamlSingleton._obj = object.__new__(cls)

        return cls._obj

    def __init__(self):
        if YamlSingleton._init_flag:
            YamlSingleton._init_flag = False
            YamlSingleton.yaml_data = self.get_yaml_data()

    def get_yaml_data(self):
        '''
        获取yaml测试数据
        :return:
        '''

        yaml_file_path = file.find_obj_yaml_file()

        if not yaml_file_path:
            return '找不到yaml数据文件'

        try:
            with open(yaml_file_path, encoding='UTF-8') as f:
                dic = yaml.load(f, Loader=yaml.FullLoader)
                return dic
        except RuntimeError as e:
            exit(1)
            print('解析用例数据文件错误，或者找不到', e)

    def get_json_data(self):
        pass

__CONFIG__ = YamlSingleton().yaml_data

def parser_response(response):
    '''
    解析body 为json数据
    :param body:
    :return:
    '''
    if isinstance(response, requests.Response) and \
            operator.eq(response.headers['Content-Type'], 'application/json; charset=UTF-8'):
        return response.json()

    else:
        return response


def parsing_json_data():
    '''
    解析后台接口返回值
    :return:
    '''
    with open(create_data_file(), encoding='utf-8')as json_file:
        data = json.load(json_file)

        return data


def parsing_case_yaml_data(interface_key=None, assert_key=None, assert_value_key=None):
    '''
    解析用例数据文件
    :param interface_key: 接口key
    :param case_key: 断言key
    :param get_case_key:
    :return:
    '''
    try:

        dic = __CONFIG__

        if interface_key and \
                assert_key and \
                assert_value_key:

            if dic.__contains__(interface_key) and \
                    dic[interface_key].__contains__(assert_key) and \
                    dic[interface_key][assert_key].__contains__(assert_value_key):
                return dic[interface_key][assert_key][assert_value_key]
        elif interface_key and \
                assert_key and \
                not assert_value_key:

            if dic.__contains__(interface_key) and \
                    dic[interface_key].__contains__(assert_key):
                return dic[interface_key][assert_key]
        elif interface_key and \
                not assert_key and \
                not assert_value_key:

            if dic.__contains__(interface_key):
                return dic[interface_key]


    except RuntimeError as e:
        exit(1)
        print('解析用例数据文件错误，或者找不到', e)


def save_response_data(response):
    '''
    保持接口返回数据
    :param response: 后台返回值
    :return:
    '''
    if response.status_code == 200:
        json_key = os.path.split(parse.urlparse(response.request.url).path)[-1]
        json_value = {json_key: parser_response(response)}
    else:
        return '无法解析后台返回值', response

    old_json_data = parsing_json_data()

    old_json_data.update(json_value)

    json_data = json.dumps(old_json_data, indent=4)

    try:

        with open(create_data_file(), 'w', encoding='utf-8') as f:

            f.write(json_data)

    except RuntimeError as error:

        print('json数据写入失败', error)


def get_interface_request_data(interface_key, case_key):
    '''
    获取接口请求数据
    :param interface_key: 接口
    :param case_key: 断言key
    :return:
    '''

    return replace_json_path_value(parsing_case_yaml_data(interface_key, case_key, configKey.YAML_KEY().REQ_DATA))


def get_interface_url(interface_key):
    '''
    获取接口URL
    :param interface_key: 接口key
    :return:
    '''
    return parsing_case_yaml_data(configKey.OBJECT_HOST) + parsing_case_yaml_data(interface_key,
                                                                                  configKey.YAML_KEY().URL)


def get_interface_case_assert_data(interface_key, case_key):
    '''
    获取 接口断言参数
    :param interface_key: 接口
    :param case_key: 用例key
    :return:
    '''
    return replace_json_path_value(parsing_case_yaml_data(interface_key, case_key, configKey.YAML_KEY().ASSERT_DATA))


def get_interface_case_des(interface_key, case_key):
    '''
    获取用例说明
    :param interface_key: 接口key
    :param case_key: 用例key
    :return:
    '''
    return parsing_case_yaml_data(interface_key, case_key, configKey.YAML_KEY().DES)


def replace_json_path_value(dic):
    '''
    替换字典内jsonpath值为接口返回值
    :param dic: 替换字典
    :return:
    '''
    response_data = parsing_json_data()

    if isinstance(dic, dict):

        for key, value in dic.items():

            if operator.ne(value, str):
                value = str(value)

            if operator.ne(value.find('$'), -1):
                dic[key] = jsonpath.jsonpath(response_data, value)[0]

    return dic


if __name__ == '__main__':
    s = YamlSingleton()
    print(s.yaml_data)
    print(s.yaml_data)
    print(s.yaml_data)



#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-11 00:09
# Author : fyt
# File   : __init__.py.py

from ytApiTest.apiReq import get,post
from ytApiTest.parsingData import get_interface_url,get_interface_request_data,get_interface_case_assert_data,get_interface_case_des
from ytApiTest.apiAssert import assert_url_code,assert_body_include_value,assert_body_ep_value


#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-07 22:22
# Author : fyt
# File   : apiReq.py

import requests,os
from urllib import parse
from dingtalkchatbot.chatbot import DingtalkChatbot

from ytApiTest import configKey
from ytApiTest import parsingData


def get_url_key(url):
	'''
	获取URL key
	:param url: URL
	:return: URL最后一个路径
	'''
	return os.path.split(parse.urlparse(url).path)[-1]

def get_account_cookies():

    cookies_key = get_url_key(parsingData.parsing_case_yaml_data(configKey.ACCOUNT_URL))
    json_data = parsingData.parsing_json_data()

    if json_data.__contains__(cookies_key):

            json_data =json_data[cookies_key]
            user_id = json_data['data']['userinfo']['userId']
            session_id = json_data['data']['sessionId']

            c = 'userId={userId}; sessionId={sessionId}; ' \
                'SMclient=MicroMessenger; SMmodel=Xiaomi_MI_8;' \
                ' SMsystem=Android_8.1.0; ' \
                'SMver=7.0.3; ' \
                'SMdisplay=393x818; ' \
                'SDKVersion=2.6.1; ' \
                'weId=supermonkey-weapp-gear; ' \
                'version=1.2.0;'.format(userId=user_id,
                                        sessionId=session_id)
            h = {
                'Cookie': c
            }

            return h

def save_account_cookies():


    url = parsingData.parsing_case_yaml_data(configKey.ACCOUNT_URL)
    data = parsingData.parsing_case_yaml_data(configKey.ACCOUNT_DATA)

    return parsingData.save_response_data(requests.post(url=url, data=data))

def get(interface_key,case_key):

    url = parsingData.get_interface_url(interface_key)
    params = parsingData.get_interface_request_data(interface_key, case_key)

    response = requests.get(url=url,
                 params=params,
                 headers=get_account_cookies())

    if response.json()['rtn'] == 25 or response.json()['rtn'] == 11:

        save_account_cookies()
        get(interface_key,case_key) #会不会死循环~~
    parsingData.save_response_data(response)
    return response

def post(interface_key,case_key):

    url  = parsingData.get_interface_url(interface_key)
    data = parsingData.get_interface_request_data(interface_key, case_key)

    response = requests.post(url=url,
                             data=data,
                             headers=get_account_cookies())

    if response.json()['rtn'] == 25 or response.json()['rtn'] == 11:

        save_account_cookies()
        post(interface_key,case_key)

    else:

        parsingData.save_response_data(response)

    return response

def send_ding_talk_info(title,text):
    '''
    markdown类型
    :param title: 首屏会话透出的展示内容
    :param text: markdown格式的消息内容
    :return:
    '''

    if parsingData.parsing_case_yaml_data(configKey.DING_TALK_URL):

        url = parsingData.parsing_case_yaml_data(configKey.DING_TALK_URL)
        DingtalkChatbot(url).send_markdown(title=title,
                                                   text=text)

    else:
        print('没有找到发送钉钉群URL')